#include "appwindow.hpp"

AppWindow::AppWindow()
{
  set_title("CS488 Assignment Two");

  // A utility class for constructing things that go into menus, which
  // we'll set up next.
  using Gtk::Menu_Helpers::MenuElem;
  using Gtk::Menu_Helpers::AccelKey;
  using Gtk::Menu_Helpers::RadioMenuElem;
  using Gtk::RadioButton_Helpers::Group;
  using Gtk::Menu_Helpers::SeparatorElem;
  
  // Set up the application menu
  // The slot we use here just causes AppWindow::hide() on this,
  // which shuts down the application.
  m_menu_app.items().push_back(MenuElem("Reset", AccelKey("a"), SigC::slot(*this, &AppWindow::reset)));
  m_menu_app.items().push_back(MenuElem("Quit", AccelKey("q"), SigC::slot(*this, &AppWindow::hide)));

  Group group;

  SigC::Slot1<void, Viewer::Mode> mode_slot = SigC::slot(*this, &AppWindow::set_mode);

  m_menu_mode.items().push_back(RadioMenuElem(group, "View-Rotate", AccelKey("o"), SigC::bind(mode_slot, Viewer::VIEW_ROTATE)));
  m_menu_mode.items().push_back(RadioMenuElem(group, "View-Translate", AccelKey("n"), SigC::bind(mode_slot, Viewer::VIEW_TRANSLATE)));
  m_menu_mode.items().push_back(RadioMenuElem(group, "View-Perspective", AccelKey("p"), SigC::bind(mode_slot, Viewer::VIEW_PERSPECTIVE)));
  
  m_menu_mode.items().push_back(SeparatorElem());
  
  m_menu_mode.items().push_back(RadioMenuElem(group, "Model-Rotate", AccelKey("r"), SigC::bind(mode_slot, Viewer::MODEL_ROTATE)));
  m_menu_mode.items().push_back(RadioMenuElem(group, "Model-Translate", AccelKey("t"), SigC::bind(mode_slot, Viewer::MODEL_TRANSLATE)));
  m_menu_mode.items().push_back(RadioMenuElem(group, "Model-Scale", AccelKey("s"), SigC::bind(mode_slot, Viewer::MODEL_SCALE)));

  m_menu_mode.items().push_back(SeparatorElem());
  
  m_menu_mode.items().push_back(RadioMenuElem(group, "Viewport", AccelKey("v"), SigC::bind(mode_slot, Viewer::VIEWPORT)));

  m_menu_mode.items()[4].activate();

  // Set up the menu bar
  m_menubar.items().push_back(Gtk::Menu_Helpers::MenuElem("_Application", m_menu_app));
  m_menubar.items().push_back(Gtk::Menu_Helpers::MenuElem("_Mode", m_menu_mode));
  
  // Pack in our widgets
  
  // First add the vertical box as our single "top" widget
  add(m_vbox);

  // Put the menubar on the top, and make it as small as possible
  m_vbox.pack_start(m_menubar, Gtk::PACK_SHRINK);

  // Put the viewer below the menubar. pack_start "grows" the widget
  // by default, so it'll take up the rest of the window.
  m_viewer.set_size_request(300, 300);
  m_vbox.pack_start(m_viewer);

  m_vbox.pack_start(m_status_label, Gtk::PACK_SHRINK);
  
  //set_clip_status();
  
  m_vbox.pack_start(m_status_clip_near, Gtk::PACK_SHRINK);
  m_vbox.pack_start(m_status_clip_far, Gtk::PACK_SHRINK);
  Glib::signal_timeout().connect(SigC::slot(*this, &AppWindow::set_clip_status), 25);

  show_all();
}

void AppWindow::reset() {
  m_viewer.reset_view();
  m_menu_mode.items()[4].activate();
}

bool AppWindow::set_clip_status() {
  set_clip_status_near();
  set_clip_status_far();
  return true;
}

void AppWindow::set_clip_status_near() {
  std::ostringstream oss;
  oss << "Near: " << m_viewer.get_near();
  m_status_clip_near.set_text(oss.str());
}

void AppWindow::set_clip_status_far() {
  std::ostringstream oss;
  oss << "Far: " << m_viewer.get_far();
  m_status_clip_far.set_text(oss.str());
}

void AppWindow::set_mode(Viewer::Mode aMode) {
  switch(aMode) {
  case Viewer::VIEW_ROTATE:
    m_status_label.set_text("Mode: View-Rotate");
    m_viewer.set_mode(aMode);
    break;
  case Viewer::VIEW_TRANSLATE:
    m_status_label.set_text("Mode: View-Translate");
    m_viewer.set_mode(aMode);
    break;
  case Viewer::VIEW_PERSPECTIVE:
    m_status_label.set_text("Mode: View-Perspective");
    m_viewer.set_mode(aMode);
    break;
  case Viewer::MODEL_ROTATE:
    m_status_label.set_text("Mode: Model-Rotate");
    m_viewer.set_mode(aMode);
    break;
  case Viewer::MODEL_TRANSLATE:
    m_status_label.set_text("Mode: Model-Translate");
    m_viewer.set_mode(aMode);
    break;
  case Viewer::MODEL_SCALE:
    m_status_label.set_text("Mode: Model-Scale");
    m_viewer.set_mode(aMode);
    break;
  case Viewer::VIEWPORT:
    m_status_label.set_text("Mode: Viewport");
    m_viewer.set_mode(aMode);
    break;
  }
}
