#ifndef CS488_VIEWER_HPP
#define CS488_VIEWER_HPP

#include <gtkmm.h>
#include <gtkglmm.h>
#include "algebra.hpp"
#include "a2.hpp"

// The "main" OpenGL widget
class Viewer : public Gtk::GL::DrawingArea {
public:

  enum Mode {
    VIEW_ROTATE,
    VIEW_TRANSLATE,
    VIEW_PERSPECTIVE,
    MODEL_ROTATE,
    MODEL_TRANSLATE,
    MODEL_SCALE,
    VIEWPORT
  };

  Viewer();
  virtual ~Viewer();

  // A useful function that forces this widget to rerender. If you
  // want to render a new frame, do not call on_expose_event
  // directly. Instead call this, which will cause an on_expose_event
  // call when the time is right.
  bool invalidate();

  // Set the parameters of the current perspective projection using
  // the semantics of gluPerspective().
  void set_perspective(double fov, double aspect,
                       double near, double far);

  // Restore all the transforms and perspective parameters to their
  // original state. Set the viewport to its initial size.
  void reset_view();

  void set_mode(Mode aMode);
  double get_near();
  double get_far();

protected:

  // Events we implement
  // Note that we could use gtkmm's "signals and slots" mechanism
  // instea*SCREEN_SCALEd, but for many classes there's a convenient member
  // function one just needs to define that'll be called with the
  // event.

  // Called when GL is first initialized
  virtual void on_realize();
  // Called when our window needs to be redrawn
  virtual bool on_expose_event(GdkEventExpose* event);
  // Called when the window is resized
  virtual bool on_configure_event(GdkEventConfigure* event);
  // Called when a mouse button is pressed
  virtual bool on_button_press_event(GdkEventButton* event);
  // Called when a mouse button is released
  virtual bool on_button_release_event(GdkEventButton* event);
  // Called when the mouse moves
  virtual bool on_motion_notify_event(GdkEventMotion* event);

private:

  Mode mode;
  Point2D point1;
  Point2D point2;
  Point2D viewport_coord[2];
  Point3D cube_vertex[8];
  bool button_down[3];
  double near_clip;
  double far_clip;
  double fov;
  double max_near_clip;
  double max_far_clip;
  
  Point3D scale;
  Matrix4x4 m_model;
  Matrix4x4 m_view;
  Matrix4x4 m_projection;
  
  double my_min(double a, double b);
  double my_max(double a, double b);
  void my_draw_line(Point3D a, Point3D b);
  void draw_model_gnomon();
  void draw_modified_cube();
  void draw_view_gnomon();
  void draw_viewport();
  void reset_view_no_invalidate();
  Point2D get_view_center();
  double get_view_height();
  double get_view_width();
};

#endif
